/* IBM Confidential
 *
 * OCO Source Materials
 *
 * 5724-U18
 *
 * (C) COPYRIGHT IBM CORP. 2011,2014
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
/*
 Copyright (c) 2011 Tom Carden, Steve Coast, Mikel Maron, Andrew Turner, Henri Bergius, Rob Moran, Derek Fowler, Gary Gale
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Mapstraction nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

;

/**
 * This is the implementation of Map.js for Maximo Spatial maps.
 */

require(
		{
			packages : [
				{
					name : "esri",
					location : window.URL_ESRI
				}
			]
		},
		[
				"dojo/_base/declare", "dojo/parser", "dojo/ready", "dojox/layout/ResizeHandle", "dojo/query",
				"dojo/dnd/Moveable", "dojo/dom-geometry", "dojo/dom-style", "dojo/promise/all","dijit/Tooltip", "dijit/_WidgetBase",
				"ibm/tivoli/fwm/mxmap/Map", "ibm/tivoli/fwm/mxmap/impl/polyline/MaximoSpatialPolyline",
				"ibm/tivoli/fwm/mxmap/impl/point/MaximoSpatialLatLonPoint",
				"ibm/tivoli/fwm/mxmap/impl/marker/MaximoSpatialMarker",
				"ibm/tivoli/fwm/mxmap/impl/boundingbox/MaximoSpatialBoundingBox",
				"ibm/tivoli/fwm/mxmap/impl/geocoder/MaximoSpatialGeocoder",
				"ibm/tivoli/fwm/mxmap/impl/MaximoSpatialMxn", "ibm/tivoli/fwm/mxmap/impl/MaximoSpatialMover",
				"ibm/tivoli/fwm/mxmap/InfoWindow", "esri/symbols/SimpleFillSymbol", "esri/symbols/SimpleLineSymbol",
				"ibm/tivoli/fwm/mxmap/LoadEsriPlugin!esri/symbols/SimpleMarkerSymbol", "esri/dijit/Popup",
				"esri/geometry/Extent", "esri/SpatialReference", "esri/map", "esri/request", "dojo/dom-construct",
				"esri/arcgis/utils", "dojo/_base/lang", "ibm/tivoli/fwm/mxmap/linear/Linear",
				"ibm/tivoli/fwm/mxmap/LoadEsriPlugin!esri/Color", "ibm/tivoli/fwm/mxmap/LoadEsriPlugin!esri/graphic",
				"ibm/tivoli/fwm/mxmap/LoadEsriPlugin!esri/geometry/geometryEngine",
				"ibm/tivoli/fwm/mxmap/LoadEsriPlugin!esri/toolbars/navigation",
				"ibm/tivoli/fwm/mxmap/LoadEsriPlugin!esri/geometry/Point", "esri/geometry/Polygon"
		],
		function ( declare, parser, ready, ResizeHandle, query, Moveable, domGeom, domStyle, all, Tooltip, _WidgetBase,
				MxMap, MaximoSpatialPolyline, MaximoSpatialLatLonPoint, MaximoSpatialMarker, MaximoSpatialBoundingBox,
				MaximoSpatialGeocoder, MaximoSpatialMxn, MaximoSpatialMover, InfoWindow, SimpleFillSymbol,
				SimpleLineSymbol, SimpleMarkerSymbol, Popup, Extent, SpatialReference, Map, esriRequest, domConstruct,
				arcgisUtils, lang, Linear, Color, Graphic, geometryEngine, Navigation, Point, Polygon ) {

			declare(
					"ibm.tivoli.fwm.mxmap.impl.MaximoSpatial",
					ibm.tivoli.fwm.mxmap.Map,
					{

						validateUrlToken : null,
						refreshMapHandle : null,
						linear : null,
						latLongWkID: 4326,
						firstLayerSpatialReference: null,
						centerWithNoSpatialReference : null,
						constructor : function ( options ) {
							this.providerName = "maximospatial";
							this._alreadyFixedStartingMboZoom = false;
							this.linear = new ibm.tivoli.fwm.mxmap.linear.Linear( {
								map : this
							} );
						},
						/**
						 * MaximoSpatial does not support traffic layer
						 */
						allowsTrafficLayer : function () {
							return false;
						},
						/**
						 * There may be some spatial specific configurations
						 */
						_getCustomInitOptions : function () {
							if ( this.customInitialMapOptions ) {
								return this.customInitialMapOptions.spatial;
							}
							log.info( "no custom configuration" );
							return {};
						},
						// check if there is a valid configuration
						createMap : function ( options ) {
							if ( !options.mapConf.SPATIAL ) {
								console.error( "no spatial conf" );
								this.maximo = new ibm.tivoli.fwm.mxmap.MaximoIntegration( options );
								this.getMaximo().showMessage( "mapcontrol", "nospatialmapconf" );

							} else {
								var spatialConf = options.mapConf.SPATIAL;
								if ( spatialConf.bingMaps != true && spatialConf.openStreetMap != true
										&& ( spatialConf.mapservices == null || spatialConf.mapservices.length == 0 )
										&& !spatialConf.webMap.webMapDefined ) {
									this.maximo = new ibm.tivoli.fwm.mxmap.MaximoIntegration( options );
									this.getMaximo().showMessage( "plussmap", "no_mapservices_configured" );
								} else {
									if ( this.refreshMapHandle ) {
										dojo.disconnect( this.refreshMapHandle );
									}
									this.refreshMapHandle = dojo.subscribe( "refreshMapListener", dojo.hitch( this,
											function ( currentMbo ) {
												this.mapConf.currentMbo = currentMbo;
												this.mapConf.plussIsGis = currentMbo.gisdata.PLUSSISGIS;
												var providerMarkerLayer = this.map.getLayer( "providerMarkerLayer" );
												if ( providerMarkerLayer ) {
													providerMarkerLayer.clear();
												}
												this.autoRefreshMap();
											} ) );
									this.inherited( arguments );
								}
							}
						},
						/**
						 * Loads the initial options for the mapstraction map creation.<Br>
						 * Returns the route url service, geocode url service, initial extent and specific spatial conf.
						 */
						_getInitOptions : function () {

							var options = {
								route : this.mapConf.route,
								geocode : this.mapConf.geocode,
								initialExtent : this._getInitialLocation(),
								lastUserLocationData : this.userSessionManager.getLastUserLocation(),
								spatial : this.mapConf.SPATIAL,
								isMobile : this.isMobile,
								defaultLenUnit : this.getDefaultLengthUnit()
							};

							return options;
						},

						_getDefaultInitialLocation : function () {
							return {};
						},
						/**
						 * Returns the geocode configuration url and rev geocode address format
						 */
						getGeoCoderConf : function () {
							var options = {
								url : this.mapConf.geocode,
								revGeocodeAddressFormat : this.mapConf.revgeocodeaddressformat
							};
							return options;
						},
						/**
						 * Esri maps is compound of several mapservices. Some of them may fail to load but the map can
						 * be used. Code tolerates failures but at the end shows a message with all mapservices that
						 * failed to load.
						 */
						_checkMapLoadedCorrectly : function () {
							var _failedLayers = this.getFailedLayers();
							if ( _failedLayers && _failedLayers.length > 0 ) {
								// plussmap,failed_loading_layer
								console.log( "_failedLayers", _failedLayers );
								var params = [
										_failedLayers[ 0 ].name, _failedLayers[ 0 ].error.message
								];
								// we use the same maximo spatial message.
								this.getMaximo().showMessage( "plussmap", "failed_loading_layer", params );
							}
						},
						/**
						 * In order to support Linked records we need to intercept the center on mbo to not only rely on
						 * X/Y but also on Featureclasses.
						 */
						centerOnMbo : function ( mboInfo, zoomlevel ) {
							// Using the MXRecord wrapper to handle mboInfo's properties
							var mxRec = new ibm.tivoli.fwm.mxmap.MXRecord( {
								mboInfo : mboInfo,
								map : this
							} );

							var spatialAutolocate = this._isAutolocateBasedOnSpatial( mboInfo );
							// Added a condition to only enter this if clause if the record
							// is not an LBS record. LBS records must be handled by the base method
							if ( ( ( this._isBasedOnSpatial( mboInfo ) == true ) || ( spatialAutolocate == true ) )
									&& !mxRec.useLBSData() ) {
								var fct = function ( args ) {
									try {
										var point = this._updateMboInfo( mboInfo, args );
										this._centerOnPoint( point, zoomlevel );
									} catch ( e ) {
										console.error( "No shape found in feature!", args, e );
										this.showWarningMessage( "map", "linkedfeaturenoshapefound", null );
									}
								};
								if ( spatialAutolocate == true ) {
									this._queryMapsServiceForMboGeometry( mboInfo.autolocate, dojo.hitch( this, fct ) );
								} else {
									this._queryMapsServiceForMboGeometry( mboInfo, dojo.hitch( this, fct ) );
								}
							} else {
								this.inherited( arguments );
							}

						},
						// Get the center screen point based on feature's geometry
						getFeatureCenterPoint : function ( feature ) {
							var geometry = feature.geometry;
							var point = feature.geometry;
							if ( geometry.type == 'polyline' ) {
								var len = null;
								var wkid = feature.geometry.spatialReference.wkid;
								// WKID 4326 and 3857 consider the curvature of the earth
								try {
									if ( wkid == 4326 || wkid == 3857 ) {
										len = geometryEngine.geodesicLength( feature.geometry );
									} else {
										len = geometryEngine.planarLength( feature.geometry );
									}
									if ( len != null && len != undefined && len > 0 ) {
										if (this.map) {
											point = this.linear.getPointByDistance( feature.geometry.paths[ 0 ], len / 2);
										} else {
											point = this.linear.getPointByDistance( feature.geometry.paths[ 0 ], len / 2, feature.geometry.spatialReference);
										}
										if ( point == null || point == undefined ) {
											throw "It is not possible to calculate the intermediate point for the linked feature";
										}
									} else {
										if ( typeof geometry.getExtent == 'function' && geometry.getExtent() ) {
											point = geometry.getExtent().getCenter();
										}

									}
								} catch ( e ) {
									console
											.error(
													"Error when trying to calculate the feature length using its geometry, using the first point (paths or rings) ",
													feature.geometry );
									console.error( e );
									if ( geometry.paths ) {
										var path = geometry.paths[ 0 ][ 0 ];
										point = new Point( path[ 0 ], path[ 1 ], new esri.SpatialReference(
												geometry.spatialReference ) );
									} else {
										if ( geometry.rings ) {
											var ring = geometry.rings[ 0 ][ 0 ];
											point = new Point( ring[ 0 ], ring[ 1 ], new esri.SpatialReference(
													geometry.spatialReference ) );
										}
									}
								}

							} else {
								if ( typeof geometry.getExtent == 'function' && geometry.getExtent() ) {
									point = geometry.getExtent().getCenter();
								} else {
									if ( geometry.paths ) {
										var path = geometry.paths[ 0 ][ 0 ];
										point = new Point( path[ 0 ], path[ 1 ], new esri.SpatialReference(
												this.spatialReference ) );
									} else {
										if ( geometry.rings ) {
											var ring = geometry.rings[ 0 ][ 0 ];
											point = new Point( ring[ 0 ], ring[ 1 ], new esri.SpatialReference(
													this.spatialReference ) );
										}
									}
								}

							}
							return point;
						},
						/**
						 * Based on a esri feature this method converts it on a mxn.LatLonPoint
						 * 
						 * @param feature
						 * @returns
						 */
						_getLatLngFromFeature : function ( feature ) {
							if ( !feature ) {
								console.error( "feature is empty", feature );
								return null;
							}
							var point = this.getFeatureCenterPoint( feature );
							var lng = point.x;
							var lat = point.y;
							var sr = point.spatialReference.wkid;

							var point = this.latLng( lat, lng, sr );
							return point;
						},
						/**
						 * Returns the layer info for a mboname/featureclass mapping.
						 * 
						 * @param mboName
						 * @param featureclassName
						 * @returns {Array}
						 */
						_getMboObjectLayers : function ( mboName, featureclassName ) {
							var conf = this.mapConf.SPATIAL;
							var layerInfo = [];
							for ( var i in conf.linkableInfo ) {
								var service = conf.linkableInfo[ i ];
								for ( var j in service.layers ) {
									var layer = service.layers[ j ];
									if ( layer.parent == mboName && layer.child == featureclassName ) {
										layerInfo.push( {
											url : service.url + "/" + layer.layerId,
											mapServerUrl : service.url,
											layer : layer,
											serviceName: service.serviceName

										} );

									}
								}
							}
							return layerInfo;
						},
						/**
						 * Checks if mbo does not have a x/y but has a feature class (is linked)
						 * 
						 * @param mboInfo
						 * @returns {Boolean}
						 */
						_isBasedOnSpatial : function ( mboInfo ) {
							return ( mboInfo && mboInfo.gisdata && mboInfo.gisdata.PLUSSISGIS == true )
									|| ( mboInfo && mboInfo.gisdata && mboInfo.gisdata.FROMFC == true );
						},
						/**
						 * Checks if this mbo is autolocated by an entity that has a featureclass (is linked)
						 * 
						 * @param mboInfo
						 * @returns
						 */
						_isAutolocateBasedOnSpatial : function ( mboInfo ) {
							if ( this._isBasedOnSpatial( mboInfo ) == false ) {
								return this._isBasedOnSpatial( mboInfo.autolocate );
							}
							return false;
						},
						/**
						 * Based on a mbo it can retrieve the mxn.LatLngPoint based on a cached feature. Used by
						 * Routing.
						 * 
						 * @param mboInfo
						 * @returns
						 */
						getPointFromMboInfo : function ( mboInfo ) {
							var point = this._getLatLngFromFeature( mboInfo.FC );
							return point;
						},
						/**
						 * Get the Symbology parameter for feature's highlight from the file 'map-symbology-config.json'
						 */
						getHighlightSymbologyByGeometryType : function ( geometryType ) {
							var symbol = {};
							var layerConfig = null;
							if ( this.mapConf.layerConfigName != null ) {
								layerConfig = this.getSymbologyManager().getLayerConfigById(
										this.mapConf.layerConfigName );
							} else if ( this.mapConf.currentMbo != null ) {
								layerConfig = this.getSymbologyManager().getLayerConfigById(
										this.mapConf.currentMbo.mxdata.mboName );
							} else {
								layerConfig = this.getSymbologyManager().getLayerConfigById(
										this.mapConf.records[ 0 ].mxdata.mboName );
							}
							var layerHighlightSymbology = this.getSymbologyManager().getHighlightConfigByLayer(
									layerConfig );
							if ( layerHighlightSymbology ) {
								dojo.forEach( layerHighlightSymbology, function ( geomType ) {
									if ( geomType.id != null && geomType.id != "" && geomType.id == geometryType ) {
										symbol = geomType;
									}
								} );
							}
							return symbol;

						},
						/**
						 * Method to highlight the current feature based on its geometry
						 */
						highlightCurrentMbo : function ( mboInfo, queryResultInfo ) {
							if ( dojo.config.fwm.debug == true ) {
								console.log( "highlightCurrentMbo", mboInfo, queryResultInfo );
							}	
							if ( queryResultInfo.features && queryResultInfo.features[ 0 ] ) {
								var feature = queryResultInfo.features[ 0 ];
								var symbol = null;
								var geom = null;
								if ( feature.geometry ) {
									var geoType = feature.geometry.type;
									var geometrySymbol = this.getHighlightSymbologyByGeometryType( geoType );
									var styleMarker = null;
									if ( geometrySymbol != null
											&& ( geometrySymbol.id === "point" || geometrySymbol.id === "multipoint" ) ) {
										symbol = new SimpleMarkerSymbol();
										symbol.setColor( new Color( JSON.parse( geometrySymbol.symbol.color ) ) );
										symbol.setOutline( new SimpleLineSymbol( SimpleLineSymbol.STYLE_NULL,
												new Color( JSON.parse( geometrySymbol.symbol.outlineColor ) ), 5 ) );
										var style = geometrySymbol.symbol.style;
										var styleMarker = null;
										switch ( style.toUpperCase() ) {
											case "SQUARE":
												styleMarker = SimpleMarkerSymbol.STYLE_SQUARE;
												break;
											case "CIRCLE":
												styleMarker = SimpleMarkerSymbol.STYLE_CIRCLE;
												break;
											case "DIAMOND":
												styleMarker = SimpleMarkerSymbol.STYLE_DIAMOND;
												break;
											default:
												styleMarker = SimpleMarkerSymbol.STYLE_CIRCLE;
												break;
										}
										symbol.setStyle( styleMarker );
									} else if ( geometrySymbol != null
											&& ( geometrySymbol.id === "line" || geometrySymbol.id === "polyline" ) ) {
										symbol = new SimpleLineSymbol( SimpleLineSymbol.STYLE_SOLID, new Color( JSON
												.parse( geometrySymbol.symbol.color ) ),
												Number( geometrySymbol.symbol.thickness ) );
									} else if ( geometrySymbol != null && geometrySymbol.id === "polygon" ) {
										symbol = new SimpleFillSymbol().setColor(
												new Color( JSON.parse( geometrySymbol.symbol.color ) ) ).setOutline(
												null );
									} else {
										symbol = new SimpleFillSymbol().setColor( new Color( [
												255, 0, 255, 0.5
										] ) ).setOutline( null );
									}
									var geom = feature.geometry;
									var graphic = new Graphic( geom, symbol );
									mboInfo.highlightGraphic = graphic;

									var providerMarkerLayer;
									if ( this.map.getLayer( "providerHighlightLayer" ) === undefined ) {
										providerMarkerLayer = new esri.layers.GraphicsLayer( {
											id : "providerHighlightLayer"
										} );
										this.map.addLayer( providerMarkerLayer );
									} else {
										providerMarkerLayer = this.map.getLayer( "providerHighlightLayer" );
									}
									providerMarkerLayer.clear();
									providerMarkerLayer.add( graphic );
								}	
							} else {
								console.error( "no feature found to highlight!", queryResultInfo );
								throw "no feature found to highligh!";
							}
						},
						/**
						 * When we are able to load the feature classes of a mbo we cache this information to not have
						 * to query the service again next time.
						 * 
						 * @param mboInfo
						 * @param args
						 * @param callback
						 * @returns
						 */
						_updateMboInfo : function ( mboInfo, queryResultInfo, callback ) {
							if ( dojo.config.fwm.debug == true ) {
								console.log( "updatemboinfo", mboInfo, queryResultInfo );
							}
							if ( queryResultInfo.features && queryResultInfo.features[ 0 ] ) {
								var retFC = queryResultInfo.features[ 0 ];
								var spatialAutolocate = this._isAutolocateBasedOnSpatial( mboInfo );
								if ( dojo.config.fwm.debug == true ) {
									console.log( "Success", queryResultInfo );
								}
								if ( spatialAutolocate == true ) {
									mboInfo.autolocate.FC = retFC;
									mboInfo.autolocate.FROMFC = true;
								} else {
									mboInfo.FC = retFC;
									mboInfo.FROMFC = true;
								}
								var point = this._getLatLngFromFeature( retFC );
								// Add the coorditates to a known variable in the mboInfo
								// regardless of provider, type of conversion, etc, so that the lat/lng point
								// can be retrieved from the layer records
								mboInfo.pointInCurrentSR = point;
								return point;
							} else {
								console.error( "no feature found!", queryResultInfo );
								throw "no feature found!";
							}

						},
						/**
						 * check if this is a SPATIAL feature class mbo that is linked
						 */
						addMboMarker : function ( mboInfo, options, layerId ) {
							var layerInfoDataTmp = this._getLayerMappingForMbo(mboInfo);
							if (layerInfoDataTmp != undefined){
								var serviceName = layerInfoDataTmp[0].serviceName;
								var sigoptionInfo = this.map.getLayer(serviceName);
								if ((sigoptionInfo) && (sigoptionInfo.sigoptions != null)){
									for (var i = 0; i < sigoptionInfo.sigoptions.length; i++){
										var sigoptionInfoPosition = sigoptionInfo.sigoptions[i];
										if (sigoptionInfoPosition.layerid ==  mboInfo.mxdata.attributes.layerId && sigoptionInfoPosition.active==false){
											return;
										}
									}
									
								}
							}
							// Using the MXRecord wrapper to handle mboInfo's properties
							var mxRec = new ibm.tivoli.fwm.mxmap.MXRecord( {
								mboInfo : mboInfo,
								map : this
							} );

							var spatialAutolocate = this._isAutolocateBasedOnSpatial( mboInfo );
							// Added a condition to only enter this if clause if the record
							// is not an LBS record. LBS records must be handled by the base method
							if ( ( ( this._isBasedOnSpatial( mboInfo ) == true ) || ( spatialAutolocate == true ) )
									&& !mxRec.useLBSData() ) {
								// add SPATIAL logic to get coordinates.
								var fct = function ( queryResultInfo ) {
									try {
										var mapServerCallback = function ( layer ) {
											var units = null;
											if ( layer && layer.units ) {
												units = layer.units;
											} else {
												units = this.map._params.units;
											}
											this.map.spatialMapManager.loadLinearLayers( this, units );
										}

										var point = this._updateMboInfo( mboInfo, queryResultInfo );
										this.highlightCurrentMbo( mboInfo, queryResultInfo );
										this._createMarker( mxRec, options, point, layerId );
										if ( spatialAutolocate == true ) {
											this.getMaximo().getAutoLocateLabel(
													dojo.hitch( this, function ( data ) {
														var labelText = data.label;
														var labelPosition = data.labelPosition;
														var labelFontSize = data.labelFontSize;
														this.createAutoLocateLabel( labelText, labelPosition,
																labelFontSize, queryResultInfo );
													} ) );
										}
										var layerInfoData = this._getLayerMappingForMbo( mboInfo );
										if ( layerInfoData ) {
											var url = layerInfoData[ 0 ].mapServerUrl;
											this._getMapServiceInfo( url, dojo.hitch( this, mapServerCallback ) );
										} else {
											dojo.publish( "startupToolbarMapInit", [] );
										}

									} catch ( e ) {
										console.error( "No feature found!", queryResultInfo, e );
										this.showWarningMessage( "map", "linkedfeaturenofeature", null );
									}
								};
								if ( spatialAutolocate == true ) {
									this._queryMapsServiceForMboGeometry( mboInfo.autolocate, dojo.hitch( this, fct ) );
								} else {
									this._queryMapsServiceForMboGeometry( mboInfo, dojo.hitch( this, fct ) );
								}

							} else {
								this.inherited( arguments );
							}
						},
						_getLayerMappingForMbo : function ( mboInfo ) {
							var layerInfoData = null;

							if (mboInfo)
							{
								if (this.currentRecordMgr.mboInfo && this.currentRecordMgr.mboInfo.autolocate)
								{
									console.info( "autoLocaleMbo:", this.currentRecordMgr.mboInfo.autolocate.mxdata.mboName);
									layerInfoData = this._getMboObjectLayers( this.currentRecordMgr.mboInfo.autolocate.mxdata.mboName,this.currentRecordMgr.mboInfo.autolocate.mxdata.attributes.plussfeatureclass);
									
								} else {
									layerInfoData = this._getMboObjectLayers( mboInfo.mxdata.mboName,mboInfo.mxdata.attributes.plussfeatureclass );								
								}
								
								console.info( "layerInfoData:", layerInfoData);

								if ( !layerInfoData || layerInfoData.length == 0 ) {
									// tries with the extendsMboName (in case of views)
									layerInfoData = this._getMboObjectLayers( mboInfo.mxdata.extendsMboName, mboInfo.mxdata.attributes.plussfeatureclass );
									if ( !layerInfoData || layerInfoData.length == 0 ) {
										console.debug( "no layer mapping info for this mbo", mboInfo.mxdata.extendsMboName,	mboInfo.mxdata.attributes.plussfeatureclass, mboInfo );
										return;
									}
								}
								if ( layerInfoData.length > 1 ) {
									console.warn( "more than one layer mapping ", mboInfo.mxdata.mboName, mboInfo.mxdata.attributes.plussfeatureclass, layerInfoData );
								}
								
								mboInfo.layerInfoData = layerInfoData;
							}
							
							
							return layerInfoData;
						},
						/**
						 * Method to get the Layer infos, it creates a cache to reduce the number of requests
						 * Parameters: layerObject: The layer object, containing the layerInfos and the Map server URL
						 * layerId - The layer id if you need infos about the layers inside the Map Server, which will
						 * use the Map Server url + layer id (MapServer/layerId), if you need infos about the Map
						 * Server, just pass null successCallback - The success callback function errCallback- The error
						 * callback function
						 */
						getLayerAttributes : function ( layerObject, layerId, successCallback, errCallback ) {
							if ( layerId != null && layerId != undefined ) {
								if ( !layerObject || !layerObject.layerInfos ) {
									console.error( "error getting layerInfos from layer: ", layerObject );
									if ( errCallback ) {
										errCallback();
									}
								} else {
									var layerInfos = layerObject.layerInfos;
									var layerInfo = null;
									for ( var i = 0; i < layerInfos.length; i++ ) {
										if ( layerInfos[ i ].id === layerId ) {
											layerInfo = layerInfos[ i ];
										}
									}
									if ( !layerInfo.layerAttributes ) {
										var layerUrl = layerObject.url + "/" + layerId;
										this._getMapServiceInfo( layerUrl, dojo.hitch( this, function ( attributes ) {
											layerInfo.layerAttributes = attributes;
											if ( successCallback ) {
												successCallback( layerInfo.layerAttributes );
											}
										} ), errCallback );
									} else {
										if ( successCallback ) {
											successCallback( layerInfo.layerAttributes );
										}
									}
								}
							} else {
								if ( !layerObject.layerAttributes ) {
									var layerUrl = layerObject.url;
									if ( layerUrl == null || layerUrl == undefined ) {
										console.error( "error getting the layerUrl from layer: ", layerObject );
										if ( errCallback ) {
											errCallback();
										}
									} else {
										this._getMapServiceInfo( layerUrl, dojo.hitch( this, function ( attributes ) {
											layerObject.layerAttributes = attributes;
											if ( successCallback ) {
												successCallback( layerObject.layerAttributes );
											}
										} ), errCallback );
									}
								} else {
									if ( successCallback ) {
										successCallback( layerObject.layerAttributes );
									}
								}
							}

						},
						_getMapServiceInfo : function ( url, successCallback, errCallback ) {
							var requestHandle = esriRequest( {
								"url" : this.validateUrlToken.addTokenIfApplicable( url ),
								"content" : {
									"f" : "json"
								},
								"callbackParamName" : "callback"
							} );
							requestHandle.then( successCallback, lang.hitch( this, function ( error ) {
								console.error( "No MapServer layer info url: ", url );
								if ( errCallback ) {
									errCallback();
								}
							} ) );
						},
						/**
						 * Based on a mboinfo it queries for its feature class based on the MaximoSpatial configuration
						 * between the Mbo and the feature classe (gisobject)
						 * 
						 * @param mboInfo
						 * @param callback
						 * @param errCb
						 */
						_queryMapsServiceForMboGeometry : function ( mboInfo, callback, errCb ) {
							// if it's cached
							if ( mboInfo.FROMFC == true ) {
								callback( {
									features : [
										mboInfo.FC
									]
								} );
								return;
							}

							// tries first with the mboName
							var layerInfoData = this._getLayerMappingForMbo( mboInfo );
							if ( layerInfoData ) {
								this._getMBOGeometry( layerInfoData, [
									mboInfo
								], callback, errCb );
							} else {
								console.error( "No layer info" );
								if ( errCb ) {
									errCb();
								}
							}

						},
						_checkIfAnyFeaturedClass : function ( recs ) {
							for ( var i in recs ) {
								var stop = recs[ i ];
								var mxrec = new ibm.tivoli.fwm.mxmap.MXRecord( {
									mboInfo : stop,
									map : this.map
								} );

								if ( mxrec.hasSPATIALCoordinates()
										|| ( mxrec.hasGISCoordinates() == false && mxrec.hasAutolocateSpatialData() == true ) ) {
									return true;
								}
							}
						},
						showAllMboRecords : function ( records, markerOptions, zoom ) {
							if ( !records || records.length == 0 || this.mapConf.isOpenMap == true ) {
								return;
							}
							var callback = function ( mboInfoArray, totalErrors ) {
								console.log( "calledback", mboInfoArray, totalErrors );
								for ( var index in records ) {
									var mxRec = new ibm.tivoli.fwm.mxmap.MXRecord( {
										mboInfo : records[ index ],
										map : this
									} );
									if ( mxRec.hasAnyCoordinates() == true ) {
										// this.map.addMboToLayerManager(this.mboInfo,
										// markerOptions);
										this.addMboToLayerManager( records[ index ], markerOptions );
									} else {
										console.warn( "Mbo with no GIS info ", records[ index ] );
									}
								}
								if ( zoom == true ) {
									this.autoCenterAndZoom();
								}
							};
							if ( this._checkIfAnyFeaturedClass( records ) == true ) {
								this.queryMultipleRecordsAtOnce( records, dojo.hitch( this, callback ), dojo.hitch(
										this, callback ) );
							} else {
								dojo.hitch( this, callback )( [] );

							}
						},
						filterRecordsInMapView : function ( mboArray, callback, errCb ) {
							var esriMap = this.getProviderMap();
							this.queryMultipleRecordsAtOnce( mboArray, callback, errCb, esriMap.extent );
						},
						queryMultipleRecordsAtOnce : function ( mboArray, callback, errCb, onRegionGeometry ) {
							var queryHash = {};
							var totalUrls = 0;
							for ( var i = 0; i < mboArray.length; i++ ) {
								var mboInfo = mboArray[ i ];
								var spatialAutolocate = this._isAutolocateBasedOnSpatial( mboInfo );
								if ( this._isBasedOnSpatial( mboInfo ) == true || spatialAutolocate == true ) {
									var refMbo = mboInfo;
									if ( spatialAutolocate == true ) {
										refMbo = mboInfo.autolocate;
									}
									console.log( "mbo ", mboInfo );
									var layerInfoData = this._getLayerMappingForMbo( refMbo );
									if ( !layerInfoData ) {
										console.error( "No layer info" );
										if ( errCb ) {
											errCb();
										}
										return;
									}
									for ( var j = 0; j < layerInfoData.length; j++ ) {
										if ( queryHash[ layerInfoData[ j ].url ] == null ) {
											totalUrls++;
											queryHash[ layerInfoData[ j ].url ] = {
												info : layerInfoData[ j ],
												refMbos : [],
												mbos : []
											};
										}
										queryHash[ layerInfoData[ j ].url ].refMbos.push( refMbo );
										queryHash[ layerInfoData[ j ].url ].mbos.push( mboInfo );
									}
								}
							}
							var allInRange = [];
							var totalErrors = [];
							var accumulate = function ( url, mbos, error ) {
								totalUrls--;
								console.log( "accumulate", url, mbos, error, totalUrls );
								allInRange = allInRange.concat( mbos );
								totalErrors.push( {
									error : error,
									url : url
								} );
								if ( totalUrls == 0 ) {
									callback( allInRange, totalErrors );
								}
							};
							for ( var layerURL in queryHash ) {
								this.queryAndCache( layerURL, queryHash, dojo.hitch( this, accumulate ),
										onRegionGeometry );

							}				
						},

						queryAndCache : function ( layerURL, queryHash, callback, regionGeometry ) {
							var layerInfoData = queryHash[ layerURL ].info;
							console.log( "URL ", layerURL, layerInfoData );

							var matchFCAttributeNames = layerInfoData.layer.linkableColumnsJson;
							var matchMBOAttributeNames = layerInfoData.layer.mboColumns;
							var queryTask, query;
							layerURLQuery = this.validateUrlToken.addTokenIfApplicable( layerURL );
							queryTask = new esri.tasks.QueryTask( layerURLQuery );
							query = new esri.tasks.Query();
							query.outSpatialReference = this.map.spatialReference;
							query.returnGeometry = true;
							if ( regionGeometry != null ) {
								query.geometry = regionGeometry;
							}
							query.spatialRelationship = esri.tasks.Query.SPATIAL_REL_CONTAINS;
							query.outFields = [
								'*'
							];
							query.where = this._createWhereClause( queryHash[ layerURL ].refMbos,
									matchFCAttributeNames, matchMBOAttributeNames );
									
							var fct = function ( args ) {
								var refMbos = queryHash[ layerURL ].refMbos;
								console.log( "args", args, refMbos, layerURL );
								var aliases = {};
								if (this.mapConf.currentMbo.mxdata.mboName=="SKDPROJECT") {		
									 aliases = args.fieldAliases;
								}
								else {			
								for ( var ii in matchFCAttributeNames ) {
									for ( var kk in args.fieldAliases ) {
										var alias = args.fieldAliases[ kk ];
										if ( alias == matchFCAttributeNames[ ii ] ) {
											aliases[ matchFCAttributeNames[ ii ] ] = kk;
										} else if ( kk == matchFCAttributeNames[ ii ] ) {
											aliases[ matchFCAttributeNames[ ii ] ] = kk;
										} else if ( kk.search( "\." + matchFCAttributeNames[ ii ] ) != -1 ) {
											aliases[ matchFCAttributeNames[ ii ] ] = kk;
										}
									}

								}
								}
								var mbos = [];
								for ( var l = 0; l < args.features.length; l++ ) {
									var feature = args.features[ l ];
									// find mbo

									// If the route contains more than one instance of the same mbo,
									// we need to return all of them so the feature class (.FC property)
									// can be appended
									var mbosFound = this._getRefMbosOfFeature( refMbos, feature.attributes,
											matchFCAttributeNames, matchMBOAttributeNames, aliases );
									if ( mbosFound.length == 0 ) {
										console.log( "cannot find the mbo reference for feature", feature );
										console.log( "in mbos ", refMbos );
									}
									var me = this;
									dojo.forEach( mbosFound, function ( mbo ) {
										var whereClause = me._createWhereClause( [
											mbo
										], matchFCAttributeNames, matchMBOAttributeNames );
										console.log( "wclause was ", whereClause );
										console.log( "for         ", mbo );
										var _arg = {
											features : [
												feature
											]
										};
										me._updateQueryResultCache( layerURL, whereClause, _arg );
										mbos.push( mbo );
										mbo.FC = feature;
										mbo.FROMFC = true;
									} );
								}
								callback( layerURL, mbos );
							};
							var errfct = function ( error ) {
								console.error( "Error querying", error );
								callback( layerURL, [], error );
							};
							queryTask.execute( query, dojo.hitch( this, fct ), dojo.hitch( this, errfct ) );
						},

						_getRefMbosOfFeature : function ( _refMbos, attributes, matchFCAttributeNames,
								matchMBOAttributeNames, aliases ) {
							// If the route contains more than one instance of the same mbo,
							// we need to return all of them so the feature class (.FC property)
							// can be appended
							var mbosFound = [];
							for ( var k = 0; k < _refMbos.length; k++ ) {
								var m = _refMbos[ k ];
								console.log( "mm ", m, aliases );
								var match = false;
								for ( var ii in matchFCAttributeNames ) {
									var fcAttr = matchFCAttributeNames[ ii ];
									console.log( matchMBOAttributeNames[ ii ].toLowerCase() );
									console.log( fcAttr, aliases[ fcAttr ], attributes[ aliases[ fcAttr ] ] );
									
									var paramObj = aliases[fcAttr];
								
									if (this.mapConf.currentMbo.mxdata.mboName=="SKDPROJECT") {
										paramObj = fcAttr.name;
									}		
					
									if ( m.mxdata.attributes[ matchMBOAttributeNames[ ii ].toLowerCase() ] == attributes[paramObj] ) {
										console.log( "found mbo for feature", fcAttr, aliases[ fcAttr ],
												matchMBOAttributeNames[ ii ].toLowerCase(),
												m.mxdata.attributes[ matchMBOAttributeNames[ ii ].toLowerCase() ],
												attributes[ aliases[ fcAttr ] ] );
										match = true;
									} else {
										match = false;
										break;
									}
								}
								if ( match == true ) {
									mbosFound.push( m );
								}
							}
							return mbosFound;
						},	
						_getMBOGeometry : function ( layerInfoData, mboInfoArray, callback, errCb, index ) {
							if ( !index ) {
								index = 0;
							}
							var layerURL = layerInfoData[ index ].url;
							var matchFCAttributeNames = layerInfoData[ index ].layer.linkableColumnsJson;
							var matchMBOAttributeNames = layerInfoData[ index ].layer.mboColumns;
							var queryTask, query;
							layerURL = this.validateUrlToken.addTokenIfApplicable( layerURL );
							queryTask = new esri.tasks.QueryTask( layerURL );
							query = new esri.tasks.Query();
							/*
							 * if the map is already created, use the map spatial reference and return all fields
							 * if not, use latitude/longitude
							 */
							if (this.map) {
								query.outSpatialReference = this.map.spatialReference;
								query.outFields = ['*'];
							} else {
								if (this.firstLayerSpatialReference) {
									query.outSpatialReference = this.firstLayerSpatialReference;
								} else {
									query.outSpatialReference = new SpatialReference(this.latLongWkID);
								}
							}
							if (this.mapConf && this.mapConf.mapServiceLayerSDE && !query.outFields)
							{
								query.outFields = ['*'];
							}
							query.returnGeometry = true;
							query.where = this._createWhereClause( mboInfoArray, matchFCAttributeNames,
									matchMBOAttributeNames );
							if ( dojo.config.fwm.debug == true ) {
								console.info( 'getting mbo with query', query.where );
								console.info( 'On layer ', layerURL );
							}
							var whereClause = query.where;
							var systemRecord = mboInfoArray[ 0 ].mxdata.attributes[ "systemRecord" ];
							/*
							 * queryCacheKey contains the where cluase, if the map if already created
							 * if not, contains the where clause and the wkid of Latitude Longitude
							 * 
							 * This was done to be able to query the mbo geometry before the map was created,
							 * store the result in lat/long, and not use the same cache when we query later using 
							 * the map's spatialReference
							 */
							var queryCacheKey;
							if (this.map) {
								queryCacheKey = query.where;
							} else if (!this.firstLayerSpatialReference) {
								queryCacheKey = query.where + " : ( wkid: " + this.latLongWkID + " )";
							} else if (this.firstLayerSpatialReference){
								queryCacheKey = query.where + " : ( wkid: " + this.firstLayerSpatialReference.wkid + " )";
							}
							if ( systemRecord == "ARCGIS" ) {
								if ( this._isInCache( layerURL, queryCacheKey ) == true ) {
									callback( this._queryCache[ layerURL ][ queryCacheKey ].results );
									return;
								}

								if ( this._hasRequestedAlready( layerURL, queryCacheKey ) == true ) {
									this._updateRequestCache( layerURL, queryCacheKey, callback, errCb );
									return;
								}
							}
							this._updateRequestCache( layerURL, queryCacheKey, callback, errCb );
							var fct = function ( args ) {
								if ( !args.features || !args.features[ 0 ] ) {
									index++;
									if ( index < layerInfoData.length ) {
										console.log( "trying next layer", index );
										this._getMBOGeometry( layerInfoData, mboInfoArray, callback, errCb, index );
									} else {
										var err = {
											errormsg : "no feature found for '" + queryCacheKey + "' on " + layerURL,
											details : {
												whereClause : whereClause,
												layerURL : layerURL,
												layerInfoData : layerInfoData
											}
										};
										this._triggerQueryResultError( layerURL, queryCacheKey, args, err );
									}
								} else {
									this._updateQueryResultCache( layerURL, queryCacheKey, args );

									while ( this._queryRequestCache[ layerURL ][ queryCacheKey ].reqs.length > 0 ) {
										if ( dojo.config.fwm.debug == true ) {
											console.log( "Calling reqs",
													this._queryRequestCache[ layerURL ][ queryCacheKey ].reqs.length );
										}
										var ff = this._queryRequestCache[ layerURL ][ queryCacheKey ].reqs.pop();
										ff.success( args );
									}
									var feature = args.features[ 0 ];
									mboInfoArray[index].gisdata.lat = feature.geometry.y;
									mboInfoArray[index].gisdata.lng = feature.geometry.x;
								}

							};
							var errfct = function ( args ) {
								console.log( "Error querying for spatial feature class", args );
								var err = {
									errormsg : "Error querying for spatial feature class"
								};
								this._triggerQueryResultError( layerURL, whereClause, args, err );

							};
							queryTask.execute( query, dojo.hitch( this, fct ), dojo.hitch( this, errfct ) );
						},

						_createWhereClause : function ( mboInfoArray, matchFCAttributeJson, matchMBOAttributeNames ) {
							console.log( mboInfoArray );
							var query = "";
							for ( var j = 0; j < mboInfoArray.length; j++ ) {
								var mboInfo = mboInfoArray[ j ];
								var queryForOneMbo = "";
								for ( var i = 0; i < matchFCAttributeJson.length; i++ ) {
									var strAdd = '\'';
									var attrJson = matchFCAttributeJson[ i ];
									var attr = attrJson.name;
									var value = mboInfo.mxdata.attributes[ matchMBOAttributeNames[ i ].toLowerCase() ];

									if ( value == null || value == undefined ) {
										console.error( "No Mbo Matching Attribute: ", mboInfo.mxdata, e );
										this.showWarningMessage( "map", "linkedfeaturenomboattribute", null );
										return;
									}
									if ( isNaN( value ) ) {
										value = value.split( "'" ).join( "''" );
										if (attrJson.type != "esriFieldTypeGlobalID") {
											value = value.toUpperCase();
											attr = "Upper(" + attr + ")";
										}
										
									}

									// Replace single apostrophes for double apostrophes
									if ( value.length == 0 ) {
										queryForOneMbo += attr + ' is null AND '; // empty strings
										// from
										// Maximo are null in the database
									} else {
										// Defect 55163 - Some Work Orders (Changes, Releases etc) and Tickets (SR,
										// Incidents etc)
										// with linked feature are not displayed into map tab.
										// Cannot decide whether or not to add single quotes based on the field value.
										// The decision needs to be made based on the field type, but since we do not
										// have access to the field type (it would be possible only by changing Spatial
										// code)
										// we always include single quotes around the value. Warning: It may break when
										// using a numeric linked field and a database that is strict about field types
										// (DB2)
										// if (isNaN(value))
										// {
										queryForOneMbo += attr + '=' + strAdd + value + strAdd + ' AND ';
										// }
										// else
										// {
										// queryForOneMbo += attr + '=' + value + ' AND ';
										// }
									}
								}
								if ( queryForOneMbo.length > 0 ) {
									queryForOneMbo = queryForOneMbo.substr( 0, queryForOneMbo.length - 5 );
									if ( query.length > 0 ) {
										query += " OR ";
									}
									query += "(";
									query += queryForOneMbo;
									query += ")";
								} else {
									console.error( "no matching attributes", mboInfo, matchFCAttributeNames,
											matchMBOAttributeNames );
								}

							}
							return query;
						},
						/**
						 * Checks if we have the results in cache
						 * 
						 * @param layerURL
						 * @param whereClause
						 * @return true if it's in cache
						 */
						_isInCache : function ( layerURL, whereClause ) {
							if ( this._queryCache[ layerURL ] ) {
								if ( this._queryCache[ layerURL ][ whereClause ] ) {
									if ( ( this._queryCache[ layerURL ][ whereClause ]._timestamp - new Date()
											.getTime() ) < this._expiringCache ) {
										this._queryCache[ layerURL ][ whereClause ]._hitTimes++;
										console.log( "Cache hit!",
												this._queryCache[ layerURL ][ whereClause ]._hitTimes );
										return true;
									} else {
										console.info( "expiring cahce for " + whereClause );
										this._queryCache[ layerURL ][ whereClause ] = null;
									}
								}
							}
							return false;
						},
						/**
						 * Updates the cache for query results
						 * 
						 * @param layerURL
						 * @param whereClause
						 * @param results
						 */
						_updateQueryResultCache : function ( layerURL, whereClause, results ) {
							if ( !this._queryCache[ layerURL ] ) {
								this._queryCache[ layerURL ] = {};
							}
							this._queryCache[ layerURL ][ whereClause ] = {
								results : results,
								_hitTimes : 0,
								_timestamp : new Date().getTime()
							};
						},
						/**
						 * Buffer sequenced requests
						 * 
						 * @param layerURL
						 * @param whereClause
						 * @param callback
						 * @param errCb
						 */
						_hasRequestedAlready : function ( layerURL, whereClause ) {
							if ( this._queryRequestCache[ layerURL ]
									&& this._queryRequestCache[ layerURL ][ whereClause ] != null ) {
								return true;
							}
							return false;
						},
						_updateRequestCache : function ( layerURL, whereClause, callback, errCb ) {
							if(!whereClause){
								return;
							}
							if ( !this._queryRequestCache[ layerURL ] ) {
								this._queryRequestCache[ layerURL ] = {};
							}
							if ( !this._queryRequestCache[ layerURL ][ whereClause ] ) {
								this._queryRequestCache[ layerURL ][ whereClause ] = {
									reqs : []
								};
							}
							this._queryRequestCache[ layerURL ][ whereClause ].reqs.push( {
								success : callback,
								error : errCb
							} );
						},
						/**
						 * If any requests where buffered we need to trigger the error callback
						 */
						_triggerQueryResultError : function ( layerURL, whereClause, args, err ) {
							while ( this._queryRequestCache[ layerURL ][ whereClause ].reqs.length > 0 ) {
								var ff = this._queryRequestCache[ layerURL ][ whereClause ].reqs.pop();
								if ( ff.error ) {
									ff.error( err, args );
								} else {
									console.log( err );
								}
							}
						},
						_queryCache : {},
						_queryRequestCache : {},
						_expiringCache : 30000,
						getInitialLocationCustomInfo : function () {
							var esriMap = this.getProviderMap();
							return {
								extentData : {
									wkid : esriMap.spatialReference.wkid,
									wkt : esriMap.spatialReference.wkt,
									xmin : esriMap.extent.xmin,
									xmax : esriMap.extent.xmax,
									ymax : esriMap.extent.ymax,
									ymin : esriMap.extent.ymin

								}
							};
						},
						_setDefaultLocation : function ( location ) {
							var esriMap = this.getProviderMap();
							if ( esriMap.extentInitiallySet != true && location && location.lat && location.lng) {
									console.log("[Map] setDefaultLocation", location);
									var latLng = this.latLng(location.lat, location.lng);
									this.setCenterAndZoom(latLng, location.level);
							}
						},
						_resize : function () {
							// Spatial API is increasing the map width for some reason,
							// and create scrolling if width=100%. Width will have to be
							// reset to a smaller value.
							
							if ( !this.width ) {
								w = '99%';
							} else {
								if ( this.width.indexOf( '%' ) > -1 ) {
									var _w = parseInt( this.width.substr( 0, this.width.length - 1 ) );
									if ( _w > 99 ) {
										this.width = '99%';
									}
								}
							}
							
							if ( !this.height ) {
								h = '94%';
							} else {
								if ( this.height.indexOf( '%' ) > -1 ) {
									var _h = parseInt( this.height.substr( 0, this.height.length - 1 ) );
									if ( _h > 94 ) {
										this.height = '94%';
									}
								}
							}

							this.inherited( arguments );
						},
						_alreadyFixedStartingMboZoom : null,
						_zoomToCurrentMboRec : function ( initialLocation ) {
							if ( this._alreadyFixedStartingMboZoom != true ) {
								var handler;
								var counter = 0;
								var fct = dojo.hitch( this, function () {
									counter++;
									if ( this.state.updating != true || counter > 100 ) {
										this.currentRecordMgr.centerAndZoom( initialLocation.level );
									} else {
										setTimeout( fct, 300 );
									}
									dojo.unsubscribe( handler );
								} );
								var h2;
								var addListener = dojo.hitch( this, function () {
									dojo.unsubscribe( h2 );
									handler = dojo.subscribe( "onESRIReposition", fct );
								} );
								h2 = dojo.subscribe( this.events.mapLoaded, addListener );
							}
							this.inherited( arguments );

						},

						/* _MapProvider implementation start */

						_init : function ( element, options ) {
							var def = new dojo.Deferred();
							this.element = element;
							this.validateUrlToken = new ibm.tivoli.fwm.mxmap.security.ValidateUrlToken( this );

							esriConfig.defaults.io.alwaysUseProxy = this.mapConf.alwaysUseProxy;
							esriConfig.defaults.io.proxyUrl = dojo.config.fwm.servletBase + "/pluss/proxy.jsp";

							this.validateUrlToken.initTokenExpiretime( this.mapConf );

							/*
							 * Fixing map open in wrong place
							 */
							if ((options.spatial.webMap) && (options.spatial.webMap.webMapDefined) ) {
								this.hasWebMap = true;
							}
							var deferredFirstlayer;
							if ( options.spatial.mapservices[0] ) {
								if (!this.basemapOrder && options.spatial.mapservices[0])
								{
									this.basemapOrder = options.spatial.mapservices[0].order;
								}
								var rParams = {
										"f": "json"
								};
								var preCallBack = function(ioArgs) {
									
								    return ioArgs;

								}
								esriRequest.setRequestPreCallback(preCallBack);
                                
								var layerURLToken = this.validateUrlToken.addTokenIfApplicable( options.spatial.mapservices[0].url ); 
								
								try {
									var deferredFirstlayer = esriRequest({
										url: layerURLToken,
										handleAs: "json",
										callbackParamName: "callback",
										async: false,
										content: rParams,
										params: {
											"mapServiceInfo": options.spatial.mapservices[0]
										},
										timeout: 30000
									});

									
								} catch (e) {
									throw e;
								} 
							}
							
							// we can assure that dojo is loaded in here, that's
							// why dojo.require is not outside this block

							this._connections = [];

							var me = this;

							// prefetch openstreetmap dojo layers
							var mapInitialExtent = null;

							var popupOptions = {
								fillSymbol : new SimpleFillSymbol( SimpleFillSymbol.STYLE_SOLID, new SimpleLineSymbol(
										SimpleLineSymbol.STYLE_SOLID, new dojo.Color( [
												255, 0, 0
										] ), 2 ), new dojo.Color( [
										255, 255, 0, 0.25
								] ) ),
								markerSymbol : new SimpleMarkerSymbol({
							          "color": [255,0,191,100],
							          "size": 12,
							          "angle": -30,
							          "xoffset": 0,
							          "yoffset": 0,
							          "type": "esriSMS",
							          "style": "esriSMSCircle",
							          "outline": {
							            "color": [255,0,191],
							            "width": 1,
							            "type": "esriSLS",
							            "style": "esriSLSSolid"
							          }
							     })
							};
							var popup = new Popup( popupOptions, dojo.create( "div" ) );

							var mapOpts = {
								isZoomSlider : true,
								isPanArrows : false,
								nav : false,
								logo : false,
								infoWindow : popup,
								autoResize : true,
								resizeDelay : 100
							};
							//Deferred object for the map center calculation
							var deferredMapCenter = new dojo.Deferred();
							if ( this.mapConf.isPreviewMobile ) {
								// Limit the zoom if it is Preview Offline Map Action
								var initZoom = this.mapConf.initialLocation.initZoom;
								var finalZoom = this.mapConf.initialLocation.finalZoom;
								mapOpts.maxZoom = finalZoom;
								mapOpts.minZoom = initZoom;
							} 
							//If in Map Manager, we won't try to get the map center before creating the map
							if (this.mapConf.isInMapManager) {
								deferredMapCenter.resolve();
							} else {

							    /* extentInitialSet will be true if we find linked Mbo, Tool Persistance extent 
								 *  or session last user location
								 */
								var extentInitiallySet = false;
								var hasLastUserLocation, hasToolsPersistance;
								var lastExtentDeferred = new dojo.Deferred();
								
								//Check if the last user location is saved
								var lastUserLocation = this._getLastUserLocation(options);
								if (lastUserLocation) {
									hasLastUserLocation = true;
									lastExtentDeferred.resolve(lastUserLocation);									
								} else {
									//Get the map initial extent based in the tools persistance info
									this.maximo = new ibm.tivoli.fwm.mxmap.MaximoIntegration( options );
									var deferredToolsConfig = new dojo.Deferred();
									this.getMaximo().loadToolsConfiguration(dojo.hitch(this, function(data) {
										var persistedExtent;
										if (data != null && data != undefined) {
											if (data.result == "hasToolsConfiguration") {
												this.toolsJson = data.toolsJson;
												hasToolsPersistance = true;
											} else {
												this.toolsJson = {};
											}
											if ( this.toolsJson.saveMapExtent == "1") {
												persistedExtent = new esri.geometry.Extent(this.toolsJson.mapExtent);
											}
										};
										lastExtentDeferred.resolve(persistedExtent);
										
									}));
									
								}
								
								var mapMboCenterPromise = new dojo.Deferred();
								if (!this.hasWebMap) {
									deferredFirstlayer.then(dojo.hitch(this, function(response) {
										this.firstLayerSpatialReference = new SpatialReference(response.spatialReference);
										this._getMapInitialPosition().then(dojo.hitch(this, function(center) {
											mapMboCenterPromise.resolve(center);
										}));
									}));
								} else {
									mapMboCenterPromise.resolve();
								}
								
								//Get the center based on the Mbo(s) position
								this.centerWithNoSpatialReference = false;
								/*
								 * When we have the last extent and the linked Mbo(s) position(s) returned
								 */
								all({lastExtent: lastExtentDeferred.promise, center: mapMboCenterPromise })
								.then(dojo.hitch(this,function(results) {
									if (results.center) {
										if (!results.center.spatialReference) {
											if (results.center.sr) {
												results.center = new Point(results.center.lng, results.center.lat, new SpatialReference(results.center.sr));
											} else {
												this.centerWithNoSpatialReference = true;
											}
										} 										
										
										//If a center point is returned (1 mbo only)
										if (results.center.type === "point"){
											mapOpts.center = results.center;
											//Get layer info to determine the correct zoom level
											var mboInfo = this.currentRecordMgr.mboInfo
											var layerInfoData = this._getLayerMappingForMbo( mboInfo );
											if (this.mapConf.autoZoomToFeature == "1" && layerInfoData && layerInfoData.length > 0) {
												var layerURL = layerInfoData[0].url;
												layerURL = this.validateUrlToken.addTokenIfApplicable( layerURL );
												var requestPromise = this.getLayerInfo(layerURL);
												requestPromise.then(lang.hitch(this, function(layerDetails) {
													mapOpts.scale = this.getOptimizedScaleFromLayer(layerDetails, null);
													extentInitiallySet = true;
													deferredMapCenter.resolve();
												}))
											
											} else {
												if ((hasLastUserLocation) && (options.lastUserLocationData.level)) {
														mapOpts.zoom = options.lastUserLocationData.level;
													} else if ( (hasToolsPersistance) && (this.toolsJson.zoomLevel) ) {
														mapOpts.zoom = this.toolsJson.zoomLevel;
													} else if (this.mapConf.initialLocation){
														mapOpts.zoom = this.mapConf.initialLocation.level;
													} else {
														mapOpts.zoom = this.DEFAULT_INITIAL_ZOOM;
													}
													extentInitiallySet = true;
													deferredMapCenter.resolve();
											}

										//if an extent is returned (more then one mbo)
										} else  if (results.center.type === "extent") {
											mapOpts.extent = results.center;	
											extentInitiallySet = true;
											deferredMapCenter.resolve();
										} else if(!results.center.type){
											deferredMapCenter.resolve();
										}							
									} else {
										if ( results.lastExtent) {
											mapOpts.extent = results.lastExtent;
											extentInitiallySet = true;
										} 	
										deferredMapCenter.resolve();
									}
									
								}));
									
							}
							
							var geoSvcData = me.mapConf.geometryServiceUrl;
							if ( geoSvcData ) {
								try {
									me.esriGeometryService = new esri.tasks.GeometryService( geoSvcData );
								} catch ( e ) {
									console.warn( "[MaximoSpatial] Error creating geometry service for data",
											geoSvcData );
								}
							}

							/** ESRI Maps always break with right to left languages.* */
							dojo.style( element, "direction", "ltr" );

							console.log( "[Check to see what map type to create]" );
							if ( this.mapConf.SPATIAL.webMap.webMapDefined ) {
								var webMapID = this.mapConf.SPATIAL.webMap.webMapID;
								console.log( "[Create Web Map Started] with Web Map ID: " + webMapID );

								var createMapOptions = {
									mapOptions : mapOpts,
									usePopupManager : true, // since version 3.10
									// bingMapsKey: bingMapsKey,
									geometryServiceURL : geoSvcData,
									ignorePopups : true
								};

								var wbmapUrl = "https://www.arcgis.com/sharing/content/items/" + webMapID + "/data";
								if ( this.mapConf.SPATIAL.webMap.arcGISPortalEnabled ) {
									arcgisUtils.arcgisUrl = this.mapConf.SPATIAL.webMap.PORTALURL;
									wbmapUrl = this.mapConf.SPATIAL.webMap.PORTALURL + "/" + webMapID + "/data";
								}

								// this.validateUrlToken.setWebMapTokenIfApplicable(wbmapUrl);

								var thisObj = this;
								var preCallBack = function ( ioArgs ) {
									console.debug( "get Token Value From Web Map Credentials: " + ioArgs.url,
											ioArgs.content );
									var tokenValue = thisObj.validateUrlToken.getTokenValueForWebCreate( ioArgs.url );

									if ( tokenValue != "" ) {
										if ( ioArgs.content == null ) {
											ioArgs.content = {};
										}
										ioArgs.content.token = tokenValue;
									}
									return ioArgs;
								}

								esriRequest.setRequestPreCallback( preCallBack );

								arcgisUtils.createMap( webMapID, dojo.create( "div" ), createMapOptions ).then(
										lang.hitch( this, function ( response ) {
											console.log( "[Web Map Title] Title: ", response.itemInfo.item.title );
											//get the mbo position
											this.firstLayerSpatialReference = response.itemInfo.itemData.spatialReference;
											
											//Create the map only when we have a center defined
											deferredMapCenter.then(lang.hitch(this,function(){
												this._getMapInitialPosition().then(dojo.hitch(this, function(center) {
													if (center) {
														mapOpts.center = center;
													}
													esriRequest.setRequestPreCallback( null );
													this.map = new Map( element.id, mapOpts );
													this.map.webMapResponse = response;
													this._initAfterMapCreation( this, element, options, extentInitiallySet,
															mapInitialExtent );
													def.callback( this );
												}));
											}));
										} ), function ( error ) {
											var errorJson = dojo.toJson( error );
											console.error( 'Creation of Web Map Failed: ', errorJson );
											def.callback( me );
											me.getMaximo().showMessage( "mapcontrol", "doNotHavePermissions", [
												errorJson
											] );
										} );
							} else {
								console.log( "[Create Normal Map Started]" );
								
								//Create the map only when we have a center defined
								deferredMapCenter.then(lang.hitch(this,function(){
									this.map = new Map( element.id, mapOpts );
									this._initAfterMapCreation( this, element, options, extentInitiallySet,
											mapInitialExtent );
									def.callback( this );
								}));
							}
							return def;
						},
						/**
						* Calculate the optimized Scale for a particular layer considering the Min Scale, Max Scale and Effective Min Scale.
						*/
						getOptimizedScaleFromLayer: function(layerJsonDetails, currentScale) {
							var newScale = null;
							var scaleMeetsRange = false;
							var minScale = layerJsonDetails.minScale;
							var maxScale = layerJsonDetails.maxScale;
												
							if( minScale == 0 && maxScale == 0 )
							{
								scaleMeetsRange = true;
							}
							else if ( minScale != 0 && maxScale == 0 )
							{
								if (currentScale != null && currentScale <= minScale )
									scaleMeetsRange = true;
							}
							else if ( minScale == 0 && maxScale != 0 )
							{
								if (currentScale != null && currentScale >= maxScale)
									scaleMeetsRange = true;
							}
							else
							{
								if (currentScale != null && currentScale <= minScale && currentScale >= maxScale)
									scaleMeetsRange = true;
							}
							if ( scaleMeetsRange == true)
							{
								newScale = currentScale;
							}
							else
							{
								if(layerJsonDetails.effectiveMinScale == null || layerJsonDetails.effectiveMinScale == undefined )
								{
									newScale = (maxScale + minScale)/2;
								}else
								{
									newScale = layerAttributes.effectiveMinScale;
								}
							}
							return newScale;
						},
						/**
						 * Including the Moveable and Resizable feature for InfoWindow, it is necessary to include on
						 * 'onShow'event because before it the title domNode is not ready.
						 */
						_addMoveableAndResizableFeature : function ( me ) {
							dojo.connect( me.map.infoWindow, "onShow", this, function () {

								// Add the Moveable feature
								if ( !me.map.infoWindow._moveableHandle ) {
									var infoWindowTitle = query( ".esriPopup .title" );
									if ( infoWindowTitle != null && infoWindowTitle.length > 0 ) {
										var component = infoWindowTitle[ 0 ];
										me.map.infoWindow._moveableHandle = new Moveable( me.map.infoWindow.domNode, {
											handle : component
										} );
										var existedTooltipMoveable = dijit.byId( "tooltipMoveable" );
										if ( existedTooltipMoveable ) {
											existedTooltipMoveable.destroyRecursive( true );
										}
										new Tooltip(
												{
													id : "tooltipMoveable",
													connectId : component,
													label : ibm.tivoli.fwm.i18n.getMaxMsg( "plussmap",
															"flyoverMoveableDialog" ),
													showDelay : me.showDelayTimeTooltip
												} );
									}
								}

								// Add the Resizable feature
								if ( !me.map.infoWindow._resizableHandle ) {
									var divResizable = dojo.create( 'div', {
										id : "divIndoWindowResizable",
										"class" : "divIndoWindowResizable"
									} );
									dojo.place( divResizable, me.map.infoWindow.domNode.childNodes[ 0 ], "first" );
									var handle = dojo.create( 'div' );
									dojo.place( handle, divResizable, "last" );
									var resizer = new ResizeHandle( {
										targetId : "divIndoWindowResizable",
										activeResize : true,
										intermediateChanges : true,
										animateMethod : "combine",
										minWidth : 1,
										minHeight : 1
									}, handle );
									resizer.startup();

									var existedTooltipResize = dijit.byId( "tooltipResize" );
									if ( existedTooltipResize ) {
										existedTooltipResize.destroyRecursive( true );
									}
									new Tooltip( {
										id : "tooltipResize",
										connectId : resizer.domNode,
										label : ibm.tivoli.fwm.i18n.getMaxMsg( "plussmap", "flyoverResizeDialog" ),
										showDelay : me.showDelayTimeTooltip
									} );
									/**
									 * Wheb the resize stops the info window is resizable using the width and height
									 * provided by ResizeHandle
									 *
									dojo.subscribe( "/dojo/resize/stop", dojo.hitch( this, function ( inst ) {
										if (me.map.infoWindow._resizableHandle.targetId == inst.targetId) {
											var increaseWidth = domStyle.get( inst.targetDomNode, "width" );
											var increaseHeight = domStyle.get( inst.targetDomNode, "height" );
											me.map.infoWindow.hide();
											me.map.infoWindow.resize( increaseWidth, increaseHeight );
											me.map.infoWindow.show();
											domStyle.set( inst.targetDomNode, "height", "100%" );
										}
									} ) ); Defect 279509*/
									me.map.infoWindow._resizableHandle = resizer;
									me.map.infoWindow._resizableHandleDiv = divResizable;
								} else {
									domStyle.set( me.map.infoWindow._resizableHandleDiv, "height", "100%" );
									domStyle.set( me.map.infoWindow._resizableHandleDiv, "width", "100%" );
								}

								/**
								 * If there is no content inside the info window (user clicks in a point that there is
								 * no feature), the resize handle icon is hidden
								 */
								var content = query( ".esriViewPopup" );
								if ( content && content.length > 0 ) {
									if ( dojo.hasClass( me.map.infoWindow._resizableHandleDiv,
											"divIndoWindowResizableDisplayNone" ) ) {
										dojo.removeClass( me.map.infoWindow._resizableHandleDiv,
												"divIndoWindowResizableDisplayNone" );
									}
								} else {
									if ( !dojo.hasClass( me.map.infoWindow._resizableHandleDiv,
											"divIndoWindowResizableDisplayNone" ) ) {
										dojo.addClass( me.map.infoWindow._resizableHandleDiv,
												"divIndoWindowResizableDisplayNone" );
									}
								}
							} );

						},
						_calculateMaxZIndexValue : function () {
							var zIndexMax = 0;
							query( ".dijitDialog" ).forEach( function ( dialog, index ) {
								var display = domStyle.get( dialog, "display" );
								if ( display != "none" ) {
									var zIndexDialog = domStyle.get( dialog, "z-index" );
									if ( zIndexDialog > zIndexMax ) {
										zIndexMax = zIndexDialog;
									}
								}
							}, this );
							return zIndexMax;
						},
						/**
						 * Event to move the InfoWindow to the foreground if there are more dialogs opened, the code
						 * verified the max z-index setup and increase by 1 the z-index of the info window
						 */
						_addOnClickMoveForeground : function ( me ) {
							var clickHandler = null;
							var zIndexDefault;
							dojo.connect( me.map.infoWindow, "onShow", this, function () {
								var esriPopupWrapper = dojo.query( ".esriPopupWrapper" );
								var esriPopup = dojo.query( ".esriPopup" )[ 0 ];
								clickHandler = esriPopupWrapper.on( "click", dojo.hitch( this, function () {
									var zIndexMax = this._calculateMaxZIndexValue();
									domStyle.set( esriPopup, "z-index", parseInt( zIndexMax ) + 1 );
								} ) );

								zIndexDefault = domStyle.getComputedStyle( esriPopup ).zIndex;
								var zIndexMax = this._calculateMaxZIndexValue();
								domStyle.set( esriPopup, "z-index", parseInt( zIndexMax ) + 1 );

							} );

							dojo.connect( me.map.infoWindow, "onHide", this, function () {
								if ( clickHandler ) {
									clickHandler.remove();
								}
								domStyle.set( dojo.query( ".esriPopup" )[ 0 ], "z-index", zIndexDefault );
							} );

						},
						_initAfterMapCreation : function ( me, element, options, extentInitiallySet, mapInitialExtent ) {

							this._addMoveableAndResizableFeature( me );
							this._addOnClickMoveForeground( me );
							me.mapDivId = element.id;
							// adjust pan rate and duration to provide a better
							// user experience
							// during drag and drop of the marker
							esriConfig.defaults.map.panDuration = 500; // time
							// in
							// milliseconds;
							// default
							// panDuration:250
							esriConfig.defaults.map.panRate = 15; // refresh
							// rate of
							// zoom
							// animation;
							// default
							// panRate:25

							// var map = me.map;

							me.map.extentInitiallySet = extentInitiallySet;
							var defLenUnit = 'metric';
							if ( options.defaultLenUnit == "MILES" ) {
								defLenUnit = 'english';
							}
							var maximoIntegration = new ibm.tivoli.fwm.mxmap.MaximoIntegration( options );
							var spatialNavigation = new Navigation( me.map );
							var spatialMapManager = new ibm.tivoli.fwm.mxmap.impl.MaximoSpatialMxn( {
								map : me.map,
								mapConfig : me.mapConf,
								mapstraction : me,
								mapNavigation : spatialNavigation,
								maximoIntegration : maximoIntegration,
								mapInitialExtent : mapInitialExtent,
								lenUnit : defLenUnit,
								esriGeometryService : me.esriGeometryService
							} );
							spatialMapManager.loadMap( options.spatial );
							me.map.spatialMapManager = spatialMapManager;

							me.toWebMercatorCache = {};
							/* function to convert lat/lng to maximo spatial reference */
							/* Note: this is a GLOBAL function */
							projectPointsToWebMercator = function ( points, callbackFct, errCallback ) {
								return spatialMapManager.projectPointsToWebMercator( points, callbackFct, errCallback );
							};
							me.fromWebMercatorCache = {};
							/* function to convert x/y in maximo spatial reference to lat/lng */
							/* Note: this is a GLOBAL function */
							projectPointsFromWebMercator = function ( points, callbackFct, errCallback ) {
								return spatialMapManager
										.projectPointsFromWebMercator( points, callbackFct, errCallback );
							};
							dojo.connect( me.map, "onClick", function ( evt ) {
								me.fireMapEvent( Events.click, [
									{
										'location' : me.latLng( evt.mapPoint.y, evt.mapPoint.x ),
										'pixel' : evt.screenPoint,
										'point' : evt.mapPoint
									}
								] );
								me.map.infoWindow.hide();
							} );

							dojo.connect( dojo.byId( element.id ), "oncontextmenu", function ( evt ) {
								me.stopEventPropagation( evt );
								dojo.stopEvent( evt );
							} );

							dojo.connect( dojo.byId( element.id ), "contextmenu",
									function ( evt ) {
										me.stopEventPropagation( evt );
										dojo.stopEvent( evt );
										var fct = function ( e ) {
											var posx = 0;
											var posy = 0;
											if ( !e )
												var e = window.event;
											if ( e.pageX || e.pageY ) {
												posx = e.pageX;
												posy = e.pageY;
											} else if ( e.clientX || e.clientY ) {
												posx = e.clientX + document.body.scrollLeft
														+ document.documentElement.scrollLeft;
												posy = e.clientY + document.body.scrollTop
														+ document.documentElement.scrollTop;
											}

											return {
												x : posx,
												y : posy
											};
										};

										var x = fct( evt ).x;
										var y = fct( evt ).y;
										y -= dojo.coords( dojo.byId( me.mapDivId ) ).y;
										x -= dojo.coords( dojo.byId( me.mapDivId ) ).x;

										var pt = new esri.geometry.Point( x, y );

										var latLng = me.map.toMap( pt );
										var point = me.latLng( latLng.y, latLng.x );

										me.fireMapEvent( Events.rightclick, [
											{
												'location' : point,
												'pixel' : {
													x : x,
													y : y
												}
											}
										] );

										return true;
									} );

							if ( this.mapConf.isPreviewMobile ) {
								// Draw the region for Preview Mobile
								var isPreviewMobileMapHandle = dojo.subscribe( "mxmap.mapLoaded", dojo.hitch( this,
										function ( eventMap ) {
											dojo.unsubscribe( isPreviewMobileMapHandle );
											var map = eventMap.map;
											var extent = eventMap.mapConf.initialLocation.extent;
											var polygonJsonExtent = {
												"rings" : extent,
												"spatialReference" : eventMap.map.spatialReference
											};
											var initialExtent = new Polygon( polygonJsonExtent );
											var lat = eventMap.mapConf.initialLocation.lat;
											var lng = eventMap.mapConf.initialLocation.lng;

											map.on( 'extent-change', function ( event ) {
												var currentCenter = map.extent.getCenter();
												var currentZoom = map.getZoom();
												if ( initialExtent && !initialExtent.contains( currentCenter )
														&& event.delta && event.delta.x !== 0 && event.delta.y !== 0 ) {
													var newCenter = map.extent.getCenter();
													var centralize = false;
													map.centerAt( initialExtent.getExtent().getCenter() );
												}
											} );

											var sfs = new SimpleFillSymbol( esri.symbol.SimpleFillSymbol.STYLE_SOLID,
													new SimpleLineSymbol( esri.symbol.SimpleLineSymbol.STYLE_DASHDOT,
															new Color( [
																	255, 0, 0
															] ), 2 ), new Color( [
															255, 255, 0, 0.0
													] ) );

											var graphic = new Graphic( initialExtent, sfs );
											var previewLayer;
											if ( eventMap.map.getLayer( "previewLayer" ) === undefined ) {
												previewLayer = new esri.layers.GraphicsLayer( {
													id : "previewLayer"
												} );
												eventMap.map.addLayer( previewLayer );
											} else {
												previewLayer = eventMap.map.getLayer( "previewLayer" );
											}
											console.log( eventMap.map.graphicsLayerIds );

											previewLayer.add( graphic );

										} ) );
							}
						},

						/**
						 * destroys map implementation
						 */
						destroyMap : function () {
							if ( this.map.infoWindow ) {
								this.map.infoWindow.destroy();
							}
							if ( this.refreshMapHandle ) {
								dojo.disconnect( this.refreshMapHandle );
							}
							if ( this._connections ) {
								for ( var link in this._connections ) {
									dojo.disconnect( this._connections[ link ] );
								}
							}
							// destroy all DIJITs registered by Esri api:
							try {
								var mapDiv = dojo.byId( this.mapDivId );
								var esriWidgets = dijit.findWidgets( mapDiv );

								dojo.forEach( esriWidgets, function ( w ) {
									var id = w.id;
									if ( id.indexOf( "esri" ) > -1 ) {
										w.destroyRecursive( true );
									}
								} );
							} catch ( e ) {
								console.warn( "exception destroying map", e );
							}
						},

						getMapType : function () {
						},
						setMapType : function ( type ) {
						},

						getCenter : function () {
							var map = this.getProviderMap();
							var pt = map.extent.getCenter();
							return this.latLng( pt.y, pt.x );
						},
						setCenter : function ( point, options ) {
							var map = this.getProviderMap();
							var pt = point.toProprietary();
							if ( map.spatialMapManager.isPointInFullExtent( pt ) == false ) {
								return;
							}
							map.centerAt( pt );
						},

						getZoom : function () {
							var map = this.getProviderMap();
							// var zoom = map.getLevel();
							if ( map.getLevel() == -1 ) {
								if ( map._maximo && map._maximo.mapStartingWidth ) {
									return ( map.extent.getWidth() / map._maximo.mapStartingWidth );
								} else {
									// no way to know the current level
									return 1;
								}
							} else {
								return map.getLevel();
							}
						},
						setZoom : function ( zoom ) {
							var map = this.getProviderMap();

							map.centerAndZoom( map.extent.getCenter(), zoom );
						},

						setCenterAndZoom : function ( point, zoom ) {
							var map = this.getProviderMap();

							var pt = point.toProprietary();

							if ( map.spatialMapManager.isPointInFullExtent( pt ) == false ) {
								console.warn( "[Maximo Spatial] Map extent does not contain center point to zoom",
										point );
							}
							
							if (this.mapConf.isOpenMap && this.mapConf.autoZoomToFeature == 1){
								zoom = 0
							}

							// Adding this timer because we need to wait some time after the map loads
							// to use centerAt and centerAndZoom APIs, otherwise they won't work properly.
							var me = this;
							var _startTimerToCenterAndZoom = function () {
								if ( me._timer ) {
									me._timer.stop();
								}
								me._timer = new dojox.timing.Timer( 1000 );
								me._timer.start();
								me._timer.onTick = function () {
									me._timer.stop();
									if ( zoom ) {
										map.centerAndZoom( pt, zoom );
									} else {
										map.centerAt( pt );
									}
								};
							}();

						},
						zoomToServerInitialExtent : function (){
							var layer = this.map.getLayer( this.map.layerIds[0] );
							var initialExtent = layer.initialExtent;
							var spatialReference = new SpatialReference(layer.spatialReference);
							var extent = new Extent(initialExtent.xmin, initialExtent.ymin, initialExtent.xmax, initialExtent.ymax, spatialReference)
							this.map.setExtent(extent);
						},
						setExtentLatLng : function (latitude, longitude, level){	
							var spatialReference = this.map.spatialReference;
							var extCenterPoint =  new Point(longitude, latitude,spatialReference);
							this.map.centerAndZoom(extCenterPoint, level);	
						},
						getBoundingBox : function ( swLat, swLng, neLat, neLng ) {
							return new ibm.tivoli.fwm.mxmap.impl.boundingbox.MaximoSpatialBoundingBox( swLat, swLng,
									neLat, neLng );
						},
						getBounds : function () {
							var map = this.getProviderMap();
							var bounds = map.extent;
							return this.getBoundingBox( bounds.ymin, bounds.xmin, bounds.ymax, bounds.xmax );
						},
						setBounds : function ( bounds ) {
							var map = this.getProviderMap();
							var sw = bounds.sw;
							var ne = bounds.ne;
							var bnd = new esri.geometry.Extent( parseFloat( sw.lon ), parseFloat( sw.lat ),
									parseFloat( ne.lon ), parseFloat( ne.lat ), map.spatialReference );
							map.setExtent( bnd, true );
						},

						/*
						 * Always in WGS84
						 */
						getAllPointsFromWGS84 : function ( points, callback, errCb ) {
							if ( window.mx.esri.sr.wkid == WGS84WKID ) {
								callback( points );
							} else {
								var map = this.getProviderMap();
								var fct = function ( points ) {

									callback( points );
								};
								if (errCb == null) {
									errCb = function(errorMsgInfo) {
										this.maximo.showMessage(errorMsgInfo.msggroup,errorMsgInfo.msgkey);
										console.log(errorMsgInfo);
									}
								}
								projectPointsFromWebMercator( points, dojo.hitch( this, fct ), errCb );
							}
						},
						getAllPointsInWGS84 : function ( points, callback, errCb ) {
							if ( window.mx.esri.sr.wkid == WGS84WKID ) {
								callback( points );
							} else {
								var map = this.getProviderMap();
								var fct = function ( points ) {
									callback( points );
								};
								if (errCb == null) {
									errCb = function(errorMsgInfo) {
										this.maximo.showMessage(errorMsgInfo.msggroup,errorMsgInfo.msgkey);
										console.log(errorMsgInfo);
									}
								}
								projectPointsToWebMercator( points, dojo.hitch( this, fct ), dojo.hitch( this, errCb ));
							}
						},

						resizeTo : function ( width, height, ww, hh ) {
							var map = this.getProviderMap();
							dojo.style( dojo.byId( this.mapDivId ), "width", width );
							// Issue 12-13085
							// Issue 12-13543: Need to convert to string before attempting to use indexOf
							var strWidth = width.toString();
							if ( strWidth.indexOf( "%" ) > -1 ) {
								strWidth = "100%";
							}
							dojo.style( map.root, "width", strWidth );
							hh = hh | height;
							dojo.style( dojo.byId( this.mapDivId ), "height", hh );

							map.resize();
							map.reposition();
						},
						clearMarkerLayer : function () {
							var providerMarkerLayer = this.map.getLayer( "providerMarkerLayer" );
							if ( providerMarkerLayer ) {
								providerMarkerLayer.clear();
							}
						},
						createProviderMarker : function ( point, markerData ) {
							var map = this.getProviderMap();

							var params = {
								point : point,
								map : this,
								compId : this.compId,
								isMobile : this.isMobile
							};
							dojo.mixin( params, markerData );
							var marker = new ibm.tivoli.fwm.mxmap.impl.marker.MaximoSpatialMarker( params );
							if ( marker && markerData ) {
								marker.addData( markerData );
							}
							var pin = marker.toProprietary();
							marker.setChild( pin );

							var providerMarkerLayer;
							if ( map.getLayer( "providerMarkerLayer" ) === undefined ) {
								providerMarkerLayer = new esri.layers.GraphicsLayer( {
									id : "providerMarkerLayer"
								} );
								map.addLayer( providerMarkerLayer );
							} else {
								providerMarkerLayer = map.getLayer( "providerMarkerLayer" );
							}
							var gPin = providerMarkerLayer.add( pin );
							if ( marker.proprietary_marker_label ) {

								providerMarkerLayer.add( marker.proprietary_marker_label );
							}

							return marker;
						},
						removeMboHighligh : function ( mboInfo ) {
							var highlightGraphic = mboInfo.highlightGraphic;
							if ( highlightGraphic != null && highlightGraphic != undefined ) {
								var providerMarkerLayer = this.map.getLayer( "providerHighlightLayer" );
								if ( providerMarkerLayer ) {
									providerMarkerLayer.remove( highlightGraphic );
									highlightGraphic = null;
								}

							}
						},
						removeProviderMarker : function ( marker ) {
							var map = this.getProviderMap();
							
							var providerMarkerLayer = this.map.getLayer( "providerMarkerLayer" );
							if ( providerMarkerLayer ) {
								providerMarkerLayer.remove( marker.proprietary_marker );
							}

							//map.graphics.remove( marker.proprietary_marker );
							if ( marker.proprietary_marker_label && providerMarkerLayer) {
								providerMarkerLayer.remove( marker.proprietary_marker_label );
							}
							marker.closeBubble();
						},

						polyline : function ( params ) {
							return new ibm.tivoli.fwm.mxmap.impl.polyline.MaximoSpatialPolyline( params );
						},

						createProviderPolyline : function ( polyline, polylineData ) {
							var map = this.getProviderMap();
							
							var providerPolylineLayer;
							if ( this.map.getLayer( "providerPolylineLayer" ) === undefined ) {
								providerPolylineLayer = new esri.layers.GraphicsLayer( {
								 id : "providerPolylineLayer"
							  } );
								this.map.addLayer( providerPolylineLayer );
							} else {
								providerPolylineLayer = this.map.getLayer( "providerPolylineLayer" );
							}
							this.moveMarkerLayerToTop();
							var colorTemp = new dojo.Color( polyline.color );
							var color = new dojo.Color( [
									colorTemp.r, colorTemp.g, colorTemp.b, polyline.opacity
							] );

							var centerPoint = polyline.getAttribute( "centerPoint" );
							var radiusInKMs = polyline.getAttribute( "radiusInKMs" );
							if ( centerPoint && radiusInKMs ) {
								var params = new esri.tasks.BufferParameters();
								var pt = centerPoint.toProprietary();
								params.geometries = [
									pt
								];
								params.distances = [
									radiusInKMs
								];
								params.unit = esri.tasks.GeometryService.UNIT_KILOMETER;
								params.bufferSpatialReference = map.spatialReference;
								params.outSpatialReference = map.spatialReference;

								var onComplete = function ( geometries ) {
									var _fillColor = new dojo.Color( polyline.fillColor || "#000000" );
									_fillColor.a = ( polyline.opacity || 1.0 ) / 2.0;
									var symbol = new esri.symbol.SimpleFillSymbol(
											esri.symbol.SimpleFillSymbol.STYLE_SOLID, new esri.symbol.SimpleLineSymbol(
													esri.symbol.SimpleLineSymbol.STYLE_SOLID, color, 3 ), _fillColor );
									var graphic = new esri.Graphic( geometries[ 0 ], symbol );
									/* removes the temporary graphic */
									if ( polyline.esri_graphic ) {
										providerPolylineLayer.remove( polyline.esri_graphic );
									}
									providerPolylineLayer.add( graphic );
									polyline.esri_graphic = graphic;
								};

								var onError = function ( error ) {
									console.warn( "[MaximoSpatial] Error getting geometry to surround marker: ", error );
									if ( polyline.esri_graphic ) {
										providerPolylineLayer.remove( polyline.esri_graphic );
									}
									polyline.esri_graphic = null;

								};
								if ( this.esriGeometryService ) {
									this.esriGeometryService.buffer( params, onComplete, onError );
								}
							}
							var polygonOrLine = polyline.toProprietary();
							var graphic = null;
							var symbol = null;
							if ( polyline.closed && polyline.closed == true ) {
								// add a temporary graphic to have some data to return
								var fill = polyline.fillColor || "#000000";
								var _fillColor = new dojo.Color( fill );
								_fillColor.a = ( polyline.opacity || 1.0 );
								symbol = new esri.symbol.SimpleFillSymbol( esri.symbol.SimpleFillSymbol.STYLE_SOLID,
										new esri.symbol.SimpleLineSymbol( esri.symbol.SimpleLineSymbol.STYLE_SOLID,
												color, 3 ), _fillColor );

							} else {
								symbol = new esri.symbol.SimpleLineSymbol().setColor( color ).setWidth( polyline.width );
							}
							graphic = new esri.Graphic( polygonOrLine, symbol );
							providerPolylineLayer.add( graphic );
							polyline.esri_graphic = graphic;
							return graphic;
						},
						removeProviderPolyline : function ( polyline ) {
							var map = this.getProviderMap();
							
							var providerPolylineLayer = map.getLayer( "providerPolylineLayer" );
							
							if ( polyline.esri_graphic ) {
								providerPolylineLayer.remove( polyline.esri_graphic );
							} else {
								console.warn( "no graphics found on this polyline" );
							}
						},

						/* enables or disables the traffic layer */
						trafficLayer : null,
						setShowTraffic : function ( state ) {
							console.warn( "Traffic not implemented in this provider." );
						},

						latLng : function ( lat, lng, sr ) {
							return new ibm.tivoli.fwm.mxmap.impl.point.MaximoSpatialLatLonPoint( {
								lat : lat,
								lng : lng,
								sr : sr,
								map : this
							} );
						},

						pointToProprietary : function ( point ) {
							return new google.maps.LatLng( point.lat, point.lng );
						},

						pointFromProprietary : function ( point ) {
							return this.latLng( point.lat(), point.lng(), null );
						},

						providerGeocode : function ( callback, errorCallback, key, address ) {
							var geocoder = new ibm.tivoli.fwm.mxmap.impl.geocoder.MaximoSpatialGeocoder( callback,
									errorCallback, {
										key : key,
										map : this,
										customParams : this.getGeoCoderConf()
									} );
							geocoder.geocode( {
								address : address
							} );
						},
						providerReverseGeocode : function ( callback, errorCallback, key, lat, lng ) {
							var geocoder = new ibm.tivoli.fwm.mxmap.impl.geocoder.MaximoSpatialGeocoder( callback,
									errorCallback, {
										key : key,
										map : this,
										customParams : this.getGeoCoderConf()
									} );
							geocoder.geocode( {
								lat : lat,
								lng : lng
							} );
						},
						createAutoLocateLabel : function ( labelText, labelPosition, labelFontSize, queryResultInfo ) {

							try {
								this.map.showLabels = true;
								var feature = queryResultInfo.features[ 0 ];
								if ( dojo.config.fwm.debug == true ) {
									console.log( "createAutoLocateLabel: ", labelText, labelPosition, labelFontSize );
								}

								var autoLocateLabel = new esri.symbol.TextSymbol( labelText );
								/* label font settings */
								var font = new esri.symbol.Font();
								font.setSize( labelFontSize + "pt" );
								autoLocateLabel.setFont( font );
								var xOffSet = 15;
								var yOffSet = 4;
								
								/* label alignment */
								var verticalAlignMent = labelPosition.charAt( 0 );
								switch ( verticalAlignMent ) {
									case "L":
										autoLocateLabel.setVerticalAlignment( "top" );
										yOffSet = -yOffSet;
										break;
									case "M":
										autoLocateLabel.setVerticalAlignment( "middle" );
										yOffSet = 0;
										break;
									case "U":
										autoLocateLabel.setVerticalAlignment( "bottom" );
										yOffSet = 3*yOffSet;
										break;
								}
								var horizontalAlignMent = labelPosition.charAt( 1 );
								switch ( horizontalAlignMent ) {
									case "L":
										autoLocateLabel.setHorizontalAlignment( "right" );
										xOffSet = -xOffSet;
										break;
									case "R":
										autoLocateLabel.setHorizontalAlignment( "left" );
										break;
								}
								
								autoLocateLabel.setOffset(xOffSet, yOffSet);

								var geom = feature.geometry;
								var graphic = new Graphic( geom, autoLocateLabel );

								var labelLayer;
								if ( this.map.getLayer( "providerLabelLayer" ) === undefined ) {
									labelLayer = new esri.layers.GraphicsLayer( {
										id : "providerLabelLayer"
									} );
									this.map.addLayer( labelLayer );
								} else {
									labelLayer = this.map.getLayer( "providerLabelLayer" );

								}
								labelLayer.clear();
								labelLayer.add( graphic );

							} catch ( e ) {
								console.error( e );
							}
						},
						/**
						 * get the Position of the linked Mbo, or extent that 
						 * has all the Mbos in case of Multiple Mbos
						 */
						_getMapInitialPosition: function() {
							var deferredInitialPosition = new dojo.Deferred();
							if (this.mapConf.zoomToDataInput == true)
							{
								if (this.currentRecordMgr.hasAnyCoordinates())
								{

									console.log("[Map] Get position of current MBO record ");
									this.getMboPosition(this.currentRecordMgr.mboInfo).then(lang.hitch(this,function(centerPoint){
										deferredInitialPosition.resolve(centerPoint);
									}))
								}
								else
								{
									console.log("[Map] Get Center when there are multiple records.");
									this.getMapExtentForMbos(this.currentRecordSetControl.mboInfoArray).then(lang.hitch(this,function(centerExtent){
										deferredInitialPosition.resolve(centerExtent);
									}));
								}
							} else {
								deferredInitialPosition.resolve(null);
							}
							return deferredInitialPosition.promise;
							
						},
						getLayerInfo: function(mapServiceUrl) {
							var deferred = new dojo.Deferred();
							var layerRequest = esri.request({
								url: mapServiceUrl,
								load: dojo.hitch(this, function(arg, arg2) {
									deferred.resolve(arg);
								}),
								error: dojo.hitch(this, function(error) {
									console.error(error);
									deferred.reject();
								}),
								handleAs: "json",
								callbackParamName: "callback",
								async: false,
								content: {"f": "json"},
								timeout: 30000
							});
							return deferred.promise;
						},
						/**
						 * Get the center of the Mbo
						 * Based on the method centerOnMbo
						 */
						getMboPosition: function(mboInfo) {
							var mxRec = new ibm.tivoli.fwm.mxmap.MXRecord( {
								mboInfo : mboInfo,
								map : this
							} );
							var deferredReturn = new dojo.Deferred();
							mxRec.getGISPoint();
							var spatialAutolocate = this._isAutolocateBasedOnSpatial( mboInfo );
							// Added a condition to only enter this if clause if the record
							// is not an LBS record. LBS records must be handled by the base method
							if ( ( ( this._isBasedOnSpatial( mboInfo ) == true ) || ( spatialAutolocate == true ) )
									&& !mxRec.useLBSData() ) {
								var resolveQuery = function ( args ) {
									try {
										var updatedPoint = this._getMboPointInMap( mboInfo, args);
										queryDeferred.resolve(updatedPoint);
									} catch ( e ) {
										console.error( "No shape found in feature!", args, e );
										this.showWarningMessage( "map", "linkedfeaturenoshapefound", null );
										queryDeferred.reject();
									}
								};
								var queryDeferred = new dojo.Deferred();
								if ( spatialAutolocate == true ) {
									this._queryMapsServiceForMboGeometry( mboInfo.autolocate, 
										dojo.hitch( this, resolveQuery ), 
										dojo.hitch(this, function (error) {
											queryDeferred.reject();
									}) );
									queryDeferred.then(lang.hitch(this,function(point){
											deferredReturn.resolve(point);
										}), lang.hitch(this, function(error) {
											deferredReturn.resolve();
									}));
								} else {
									this._queryMapsServiceForMboGeometry( mboInfo, 
										dojo.hitch( this, resolveQuery ), 
										dojo.hitch(this, function (error) {
											queryDeferred.reject();
									}) );
									queryDeferred.then(lang.hitch(this,function(point){
											deferredReturn.resolve(point);
										}), lang.hitch(this, function (error) {
											deferredReturn.resolve();
									}));
								}
								return deferredReturn.promise;
							} else {
								var localgisdata = mxRec.getGISData();
								if (mxRec.getAutolocateMboInfo() != null)
								{
									localgisdata = mxRec.getAutolocateGISData();
								}
								if (dojo.config.fwm.debug == true)
								{
									console.log("Center on  ", mboInfo.mxdata.mboName, localgisdata.lat, localgisdata.lng);
									console.log("Is autolocated ", mboInfo.autolocate != null);
								}
								var point = this.latLng(localgisdata.lat, localgisdata.lng);

								if (mxRec.useLBSData())
								{
									point = mxRec.getLBSPoint();
									point.sr = WGS84WKID;
									if(this.map!=null){
									var auxFct = function(/* array of projected points */points)
									{
										points[0].zoomlevel = zoomlevel;
										deferredReturn.resolve(points[0]);
									};
									this.getAllPointsFromWGS84([ point ], dojo.hitch(this, auxFct));
									}else{
										deferredReturn.resolve(point);
									}
								}
								else
								{
									// Adding a null check to make sure that the marker is only drawn if the coordinates are not null.
									// The only possible scenario for this to happen is if the record is linked to a spatial feature class
									// and after that the user tries to see the the record on another map provider.
									// There is a MXRecord.hasAnyCoordinates() check before the code reaches here, but, if the record is
									// linked to a spatial feature class, that method will return true even if no GIS lat/lng exist for the Maximo record
									if(point.lat != null && point.lng != null)
									{
										deferredReturn.resolve(point);
									} 
								}
								return deferredReturn.promise;
								
							}
							
						},
						/**
						 * Function to move the Marker layer to the top.
						 */
						moveMarkerLayerToTop: function() {
							var mapInstance = this.map;
							var markerLayer = mapInstance.getLayer("providerMarkerLayer");
							var graphicLayersLength = mapInstance.graphicsLayerIds.length;
							if (markerLayer && graphicLayersLength>0) {
								
								mapInstance.reorderLayer(markerLayer, graphicLayersLength-1);
							}
						},
						/**
						 * Returns the initial extent when we have more than one Mbo associated
						 */
						getMapExtentForMbos: function(mboInfoArray) {
							var deferredMapExtentForMbos = new dojo.Deferred();
							if (!mboInfoArray || mboInfoArray.length == 0)
							{
								deferredMapExtentForMbos.resolve(null);
								return deferredMapExtentForMbos.promise;
							}
							if (mboInfoArray && mboInfoArray.length == 1)
							{
								this.getMboPosition(mboInfoArray[0]).then(function(point){
									deferredMapExtentForMbos.resolve(point);
								});
							}
							else
							{
								deferredMapExtentForMbos.resolve(null);	
							}
							return deferredMapExtentForMbos.promise;
						},
						/**
						 * Returns an extent based in the session's last user location
						 */
						_getLastUserLocation: function(options) {
							var mapInitialExtent;
							if ( options.lastUserLocationData && options.lastUserLocationData.customData
									&& options.lastUserLocationData.customData.extentData ) {
								var extent = options.lastUserLocationData.customData.extentData;
								var spatialReference = new SpatialReference( {
									wkid : extent.wkid,
									wkt : extent.wkt
								} );
								mapInitialExtent = new Extent( extent.xmin, extent.ymin, extent.xmax, extent.ymax,
										spatialReference );
							}
							return mapInitialExtent;
						},
						/**
						 * Returns the center of the features, given the query results
						 */
						_getMboPointInMap: function( mboInfo, queryResultInfo, callback) {
								if ( queryResultInfo.features && queryResultInfo.features[ 0 ] ) {
									var feature = queryResultInfo.features[ 0 ];

									if ( !feature ) {
										console.error( "feature is empty");
										return null;
									}
									var point = this.getFeatureCenterPoint( feature );
									
									return point;
								} else {
									console.error( "no feature found!", queryResultInfo );
									throw "no feature found!";
								}
						},	
						/**
						 * Refreshes the map dynamic layers
						 */
						refreshMap: function (refreshOptions) {
							var layersIds = this.map.layerIds;
							dojo.forEach(layersIds, dojo.hitch(this,function(layerId){
								var layer = this.map.getLayer(layerId);
								if (layer instanceof esri.layers.DynamicMapServiceLayer ){
									layer.refresh();
								}
							}));
							this.inherited(arguments);
						}
						
					} );
		} );
